/*****************************************************************************
*                                                                            *
*      COMMON OCEANOGRAPHIC DATA ACCESS SYSTEM (CODAS)                       *
*                                                                            *
*      WRITTEN BY:  RAMON CABRERA, ERIC FIRING, and JULIE RANADA             *
*                   JOINT INSTITUTE FOR MARINE AND ATMOSPHERIC RESEARCH      *
*                   1000 POPE ROAD  MSB 404                                  *
*                   HONOLULU, HI 96822                                       *
*                                                                            *
*      VERSION:     3.00                                                     *
*                                                                            *
*      DATE:        APRIL 1989                                               *
*                                                                            *
*****************************************************************************/
/*

    FILE:  prtarray.h

           General array printing functions and supporting routines.

*/
#ifndef prtarray_included
#define prtarray_included
#ifndef dbhost_included
#include "dbhost.h"          /* PROTOTYPE_ALLOWED */
#endif
#ifndef dbext_included
#include "dbext.h"           /* DOUBLE, etc. */
#endif
#if PROTOTYPE_ALLOWED
void clear_screen(void);
void check_screen_lines(int increm);
unsigned int print_array_1u    (FILE *fp, UBYTE array[], unsigned int nv,
                                int ncol, char *format);
unsigned int print_array_1s    (FILE *fp, BYTE array[], unsigned int nv,
                                int ncol, char *format);
unsigned int print_array_2u    (FILE *fp, USHORT array[], unsigned int nv,
                                int ncol, char *format);
unsigned int print_array_2s    (FILE *fp, SHORT array[], unsigned int nv,
                                int ncol, char *format);
unsigned int print_array_4     (FILE *fp, LONG array[], unsigned int nv,
                                int ncol, char *format);
unsigned int print_array_float (FILE *fp, FLOAT array[], unsigned int nv,
                                int ncol, char *format);
unsigned int print_array_double(FILE *fp, DOUBLE array[], unsigned int nv,
                                int ncol, char *format);
#else
void clear_screen();
void check_screen_lines();
unsigned int print_array_1u    ();
unsigned int print_array_1s    ();
unsigned int print_array_2u    ();
unsigned int print_array_2s    ();
unsigned int print_array_4     ();
unsigned int print_array_float ();
unsigned int print_array_double();
#endif
#endif /* ifndef prtarray_included */

